import Flutter
import CoreNFC

@available(iOS 11.0, *)
public class SwiftNfcManagerPlugin: NSObject, FlutterPlugin, NFCNDEFReaderSessionDelegate {

    let channel: FlutterMethodChannel
    var session: NFCReaderSession?

    private init(_ channel: FlutterMethodChannel) {
        self.channel = channel
    }

    public static func register(with registrar: FlutterPluginRegistrar) {
        let channel = FlutterMethodChannel(name: "plugins.flutter.io/nfc_manager", binaryMessenger: registrar.messenger())
        let instance = SwiftNfcManagerPlugin(channel)
        registrar.addMethodCallDelegate(instance, channel: channel)
    }

    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        switch (call.method) {
        case "startSession":
            handleStartSession(result)
            break
        case "stopSession":
            handleStopSession(result)
            break
        default:
            result(FlutterMethodNotImplemented)
        }
    }

    private func handleStartSession(_ result: FlutterResult) {
        session = NFCNDEFReaderSession(delegate: self, queue: nil, invalidateAfterFirstRead: false)
        session!.begin()
        result(true)
    }

    private func handleStopSession(_ result: FlutterResult) {
        if let session = self.session {
            session.invalidate()
            self.session = nil
        }
        result(true)
    }

    public func readerSession(_ session: NFCNDEFReaderSession, didInvalidateWithError error: Error) {
        // TODO
    }

    public func readerSession(_ session: NFCNDEFReaderSession, didDetectNDEFs messages: [NFCNDEFMessage]) {
        if (messages.count > 1) {
            NSLog("Detected multiple messages. The plugin will handle only the first message.")
        }
        // Keep the arguments structure compatible with features included in iOS13 that will be released soon.
        let key = NSUUID().uuidString
        let arguments: Dictionary<String, Any?> = ["key": key, "ndef": ["cachedNdefMessage": utilDictionalyFromMessage(messages.first!)]]
        channel.invokeMethod("onTagDiscovered", arguments: arguments)
    }

    private func utilDictionalyFromMessage(_ message: NFCNDEFMessage) -> Dictionary<String, Any?> {
        return [
            "records": message.records.map { utilDictionaryFromPayload($0) }
        ]
    }

    private func utilDictionaryFromPayload(_ payload: NFCNDEFPayload) -> Dictionary<String, Any?> {
        return [
            "id": payload.identifier,
            "payload": payload.payload,
            "type": payload.type,
            "typeNameFormat": payload.typeNameFormat.rawValue,
        ]
    }
}

